#! /bin/sh
# $Id: xmalloc.t,v 1.2 2001/02/08 02:20:14 rra Exp $
#
# Test suite for xmalloc and friends.

# The count starts at 1 and is updated each time ok is printed.  printcount
# takes "ok" or "not ok".
count=1
printcount () {
    echo "$1 $count"
    count=`expr $count + 1`
}

# Run a program expected to succeed, and print ok if it does.
runsuccess () {
    output=`$xmalloc "$1" "$2" 2>&1 >/dev/null`
    if test $? = 0 && test -z "$output" ; then
        printcount "ok"
    else
        printcount "not ok"
        echo "  $output"
    fi
}

# Run a program expected to fail and make sure it fails with an exit status
# of 2 and the right failure message.  Strip the colon and everything after
# it off the error message since it's system-specific.
runfailure () {
    output=`$xmalloc "$1" "$2" 2>&1 >/dev/null`
    status=$?
    output=`echo "$output" | sed 's/:.*//'`
    if test $status = 1 && test x"$output" = x"$3" ; then
        printcount "ok"
    else
        printcount "not ok"
        echo "  saw: $output"
        echo "  not: $3"
    fi
}

# Find where the helper program is.
xmalloc=xmalloc
for file in ./xmalloc lib/xmalloc ../xmalloc ; do
    [ -x $file ] && xmalloc=$file
done

# Total tests.
echo 21

# First run the tests expected to succeed.
runsuccess "c" "24"
runsuccess "c" "128000"
runsuccess "m" "21"
runsuccess "m" "128000"
runsuccess "m" "0"
runsuccess "r" "21"
runsuccess "r" "128000"
runsuccess "s" "21"
runsuccess "s" "128000"

# Now limit our memory to 96KB and then try the large ones again, all of
# which should fail.
ulimit -d 96
runfailure "m" "128000" \
    "failed to malloc 128000 bytes at lib/xmalloc.c line 28"
runfailure "r" "128000" \
    "failed to realloc 128000 bytes at lib/xmalloc.c line 49"
runfailure "s" "64000" \
    "failed to strdup 64000 bytes at lib/xmalloc.c line 73"
runfailure "c" "128000" \
    "failed to calloc 128000 bytes at lib/xmalloc.c line 93"

# Check our custom error handler.
runfailure "M" "128000" "malloc 128000 lib/xmalloc.c 28"
runfailure "R" "128000" "realloc 128000 lib/xmalloc.c 49"
runfailure "S" "64000" "strdup 64000 lib/xmalloc.c 73"
runfailure "C" "128000" "calloc 128000 lib/xmalloc.c 93"

# Check the smaller ones again just for grins.
runsuccess "c" "24"
runsuccess "m" "21"
runsuccess "r" "32"
runsuccess "s" "64"
